@@ -17,9 +17,15 @@
 // All changes made under the Poppler project to this file are licensed
 // under GPL version 2 or later
 //
-// Copyright (C) 2007 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2007, 2010, 2012 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2008 Boris Toloknov <tlknv@yandex.ru>
 // Copyright (C) 2008 Tomas Are Haavet <tomasare@gmail.com>
+// Copyright (C) 2010 OSSD CDAC Mumbai by Leena Chourey (leenac@cdacmumbai.in) and Onkar Potdar (onkar@cdacmumbai.in)
+// Copyright (C) 2011 Joshua Richardson <jric@chegg.com>
+// Copyright (C) 2011 Stephen Reichling <sreichling@chegg.com>
+// Copyright (C) 2012 Igor Slepchin <igor.slepchin@gmail.com>
+// Copyright (C) 2012 Luis Parravicini <lparravi@gmail.com>
+// Copyright (C) 2013 Julien Nabet <serval2412@yahoo.fr>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -27,13 +33,15 @@
 //========================================================================
 
 #include "HtmlFonts.h"
+#include "HtmlUtils.h"
 #include "GlobalParams.h"
 #include "UnicodeMap.h"
+#include "GfxFont.h"
 #include <stdio.h>
 
  struct Fonts{
-    char *Fontname;
-    char *name;
+    const char *Fontname;
+    const char *name;
   };
 
 const int font_num=13;
@@ -57,6 +65,7 @@ static Fonts fonts[font_num+1]={
 
 #define xoutRound(x) ((int)(x + 0.5))
 extern GBool xml;
+extern GBool fontFullName;
 
 GooString* HtmlFont::DefaultFont=new GooString("Times"); // Arial,Helvetica,sans-serif
 
@@ -75,10 +84,10 @@ GooString *HtmlFontColor::convtoX(unsigned int xcol) const{
   char tmp;
   unsigned  int k;
   k = (xcol/16);
-  if ((k>=0)&&(k<10)) tmp=(char) ('0'+k); else tmp=(char)('a'+k-10);
+  if (k<10) tmp=(char) ('0'+k); else tmp=(char)('a'+k-10);
   xret->append(tmp);
   k = (xcol%16);
-  if ((k>=0)&&(k<10)) tmp=(char) ('0'+k); else tmp=(char)('a'+k-10);
+  if (k<10) tmp=(char) ('0'+k); else tmp=(char)('a'+k-10);
   xret->append(tmp);
  return xret;
 }
@@ -97,10 +106,12 @@ GooString *HtmlFontColor::toString() const{
   return tmp;
 } 
 
-HtmlFont::HtmlFont(GooString* ftname,int _size, GfxRGB rgb){
+HtmlFont::HtmlFont(GfxFont *font, int _size, GfxRGB rgb){
   //if (col) color=HtmlFontColor(col); 
   //else color=HtmlFontColor();
   color=HtmlFontColor(rgb);
+  GooString* ftname=font->getName();
+  if (!ftname) ftname = getDefaultFont();
 
   GooString *fontname = NULL;
 
@@ -118,13 +129,22 @@ HtmlFont::HtmlFont(GooString* ftname,int _size, GfxRGB rgb){
   size=(_size-1);
   italic = gFalse;
   bold = gFalse;
+  rotOrSkewed = gFalse;
+
+  if (font->isBold() || font->getWeight() >= GfxFont::W700) bold=gTrue;
+  if (font->isItalic()) italic=gTrue;
 
   if (fontname){
-    if (strstr(fontname->lowerCase()->getCString(),"bold"))  bold=gTrue;
-    
-    if (strstr(fontname->lowerCase()->getCString(),"italic")||
-	strstr(fontname->lowerCase()->getCString(),"oblique"))  italic=gTrue;
-    
+    if (!bold && strstr(fontname->lowerCase()->getCString(),"bold")) {
+		bold=gTrue;
+    }
+
+    if (!italic &&
+	(strstr(fontname->lowerCase()->getCString(),"italic")||
+	 strstr(fontname->lowerCase()->getCString(),"oblique"))) {
+		italic=gTrue;
+    }
+
     int i=0;
     while (strcmp(ftname->getCString(),fonts[i].Fontname)&&(i<font_num)) 
 	{
@@ -136,6 +156,7 @@ HtmlFont::HtmlFont(GooString* ftname,int _size, GfxRGB rgb){
     pos = font_num; 
   if (!DefaultFont) DefaultFont=new GooString(fonts[font_num].name);
 
+  rotSkewMat[0] = rotSkewMat[1] = rotSkewMat[2] = rotSkewMat[3] = 0;
 }
  
 HtmlFont::HtmlFont(const HtmlFont& x){
@@ -146,6 +167,8 @@ HtmlFont::HtmlFont(const HtmlFont& x){
    pos=x.pos;
    color=x.color;
    if (x.FontName) FontName=new GooString(x.FontName);
+   rotOrSkewed = x.rotOrSkewed;
+   memcpy(rotSkewMat, x.rotSkewMat, sizeof(rotSkewMat));
  }
 
 
@@ -174,14 +197,15 @@ void HtmlFont::clear(){
 
 
 /*
-  This function is used to compare font uniquily for insertion into
+  This function is used to compare font uniquely for insertion into
   the list of all encountered fonts
 */
 GBool HtmlFont::isEqual(const HtmlFont& x) const{
-  return ((size==x.size) &&
+  return (size==x.size) &&
 	  (lineSize==x.lineSize) &&
 	  (pos==x.pos) && (bold==x.bold) && (italic==x.italic) &&
-	  (color.isEqual(x.getColor())));
+	  (color.isEqual(x.getColor())) && isRotOrSkewed() == x.isRotOrSkewed() &&
+	  (!isRotOrSkewed() || rot_matrices_equal(getRotMat(), x.getRotMat()));
 }
 
 /*
@@ -230,11 +254,11 @@ GooString* HtmlFont::HtmlFilter(Unicode* u, int uLen) {
   for (int i = 0; i < uLen; ++i) {
     switch (u[i])
       { 
-	case '"': tmp->append("&quot;");  break;
+	case '"': tmp->append("&#34;");  break;
 	case '&': tmp->append("&amp;");  break;
 	case '<': tmp->append("&lt;");  break;
 	case '>': tmp->append("&gt;");  break;
-	case ' ': tmp->append( !xml && ( i+1 >= uLen || !tmp->getLength() || tmp->getChar( tmp->getLength()-1 ) == ' ' ) ? "&nbsp;" : " " );
+	case ' ': tmp->append( !xml && ( i+1 >= uLen || !tmp->getLength() || tmp->getChar( tmp->getLength()-1 ) == ' ' ) ? "&#160;" : " " );
 	          break;
 	default:  
 	  {
@@ -266,7 +290,7 @@ GooString* HtmlFont::simple(HtmlFont* font, Unicode* content, int uLen){
 }
 
 HtmlFontAccu::HtmlFontAccu(){
-  accu=new GooVector<HtmlFont>();
+  accu=new std::vector<HtmlFont>();
 }
 
 HtmlFontAccu::~HtmlFontAccu(){
@@ -274,8 +298,8 @@ HtmlFontAccu::~HtmlFontAccu(){
 }
 
 int HtmlFontAccu::AddFont(const HtmlFont& font){
- GooVector<HtmlFont>::iterator i; 
- for (i=accu->begin();i!=accu->end();i++)
+ std::vector<HtmlFont>::iterator i; 
+ for (i=accu->begin();i!=accu->end();++i)
  {
 	if (font.isEqual(*i)) 
 	{
@@ -287,45 +311,27 @@ int HtmlFontAccu::AddFont(const HtmlFont& font){
  return (accu->size()-1);
 }
 
-// get CSS font name for font #i 
-GooString* HtmlFontAccu::getCSStyle(int i, GooString* content){
-  GooString *tmp;
-  GooString *iStr=GooString::fromInt(i);
-  
-  if (!xml) {
-    tmp = new GooString("<span class=\"ft");
-    tmp->append(iStr);
-    tmp->append("\">");
-    tmp->append(content);
-    tmp->append("</span>");
-  } else {
-    tmp = new GooString("");
-    tmp->append(content);
-  }
-
-  delete iStr;
-  return tmp;
-}
-
 // get CSS font definition for font #i 
-GooString* HtmlFontAccu::CSStyle(int i){
+GooString* HtmlFontAccu::CSStyle(int i, int j){
    GooString *tmp=new GooString();
    GooString *iStr=GooString::fromInt(i);
+   GooString *jStr=GooString::fromInt(j);
 
-   GooVector<HtmlFont>::iterator g=accu->begin();
+   std::vector<HtmlFont>::iterator g=accu->begin();
    g+=i;
    HtmlFont font=*g;
    GooString *Size=GooString::fromInt(font.getSize());
    GooString *colorStr=font.getColor().toString();
-   GooString *fontName=font.getFontName();
+   GooString *fontName=(fontFullName ? font.getFullName() : font.getFontName());
    GooString *lSize;
    
    if(!xml){
      tmp->append(".ft");
+     tmp->append(jStr);
      tmp->append(iStr);
      tmp->append("{font-size:");
      tmp->append(Size);
-     if( font.getLineSize() != -1 )
+     if( font.getLineSize() != -1 && font.getLineSize() != 0 )
      {
 	 lSize = GooString::fromInt(font.getLineSize());
 	 tmp->append("px;line-height:");
@@ -336,6 +342,29 @@ GooString* HtmlFontAccu::CSStyle(int i){
      tmp->append(fontName); //font.getFontName());
      tmp->append(";color:");
      tmp->append(colorStr);
+     // if there is rotation or skew, include the matrix
+     if (font.isRotOrSkewed()) {
+    	 const double * const text_mat = font.getRotMat();
+    	 GooString matrix_str(" matrix(");
+    	 matrix_str.appendf("{0:10.10g}, {1:10.10g}, {2:10.10g}, {3:10.10g}, 0, 0)",
+    			 text_mat[0], text_mat[1], text_mat[2], text_mat[3]);
+    	 tmp->append(";-moz-transform:");
+    	 tmp->append(&matrix_str);
+    	 tmp->append(";-webkit-transform:");
+    	 tmp->append(&matrix_str);
+    	 tmp->append(";-o-transform:");
+    	 tmp->append(&matrix_str);
+    	 tmp->append(";-ms-transform:");
+    	 tmp->append(&matrix_str);
+    	 // Todo: 75% is a wild guess that seems to work pretty well;
+    	 // We probably need to calculate the real percentage
+    	 // Based on the characteristic baseline and bounding box of current font
+    	 // PDF origin is at baseline
+    	 tmp->append(";-moz-transform-origin: left 75%");
+    	 tmp->append(";-webkit-transform-origin: left 75%");
+    	 tmp->append(";-o-transform-origin: left 75%");
+    	 tmp->append(";-ms-transform-origin: left 75%");
+     }
      tmp->append(";}");
    }
    if (xml) {
@@ -352,6 +381,7 @@ GooString* HtmlFontAccu::CSStyle(int i){
 
    delete fontName;
    delete colorStr;
+   delete jStr;
    delete iStr;
    delete Size;
    return tmp;
